#!/usr/bin/perl -w

package ME::VFS::Fuse;

use strict;
use warnings;

use Fuse;
use POSIX;

sub mount {
  my $fs = shift;
  my $mountpoint = shift;

  Fuse::main(
    mountpoint => $mountpoint,
    getattr => sub {
      my $path = shift;;
      my $Node = $fs->PathToNode($path);
      return -ENOENT() unless ($Node);
      my $Size = $Node->IsFile ? $Node->FileSize() : 0;
      return (
        $fs->UserId(),     # dev
        $Node->NodeId(),   # ino
        0755,              # mode
        1,                 # nlink
        0,                 # uid
        0,                 # gid
        0,                 # rdev
        $Size,             # size
        $Node->Modified(), # atime
        $Node->Modified(), # mtime
        $Node->Created(),  # ctime
        512,               # block size
        ceil($Size/512),   # nblocks
      );
    },
    readlink => sub { die "not implemented" },
    getdir => sub {
      my $path = shift;;
      my $Node = $fs->PathToNode($path);
      return -ENOENT() unless ($Node);
      my @res;
      # XXX - '.' and '..' ?
      push @res, map { $_->NodeName() } $Node->SubDirs();
      push @res, map { $_->NodeName() } $Node->Files();
      return (@res, 0); # errno
    },
    mknod => sub { die "not implemented" },
    mkdir => sub {
      my $path = shift;
      my $mode = shift;
      # ignore the mode
      my $Node = $fs->Mkdir($path);
      return $Node ? 0 : -EIO();
    },
    unlink => sub {
      my $path = shift;
      $fs->Delete($path);
      return 0;
    },
    rmdir => sub {
      my $path = shift;
      $fs->Delete($path);
      return 0;
    },
    rename => sub {
      my $src = shift;
      my $dest = shift;
      my $srcnode = $fs->PathToNode($src);
      return -ENOENT() unless $srcnode;

      # check dest exists for dest user
      my ($parent, $this) = $fs->ParentAndFile($dest);
      my $dstdir = $fs->PathToNode($parent);
      return -ENOENT() unless $dstdir;

      $srcnode->MoveTo(ParentNode => $dstdir, NodeName => $this);
    },
  );
}

1;
